// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// expr_test.go
package expr

import (
	"fmt"
	"strings"
	"testing"
)

func TestExpr(t *testing.T) {
	type inputType struct {
		source     string
		wantResult any
		wantErr    error
	}

	inputs := []inputType{
		/*   1 */ {`fact=func(n){(n)?{1}::{n*fact(n-1)}}; fact(5)`, int64(120), nil},
	}

	succeeded := 0
	failed := 0

	inputs1 := []inputType{
		{`f=openFile("/tmp/test2.txt"); line=readFile(f); closeFile(f); line`, "ciao", nil},
		//{`f = func(op){op()}; f(func(){2})`, int64(2), nil},
	}

	for i, input := range inputs1 {
		var expr Expr
		var gotResult any
		var gotErr error

		ctx := NewSimpleFuncStore()
		// ImportMathFuncs(ctx)
		// ImportImportFunc(ctx)
		ImportOsFuncs(ctx)
		parser := NewParser(ctx)

		logTest(t, i+1, input.source, input.wantResult, input.wantErr)

		r := strings.NewReader(input.source)
		scanner := NewScanner(r, DefaultTranslations())

		good := true
		if expr, gotErr = parser.Parse(scanner); gotErr == nil {
			gotResult, gotErr = expr.Eval(ctx)
		}

		if gotResult != input.wantResult {
			t.Errorf("%d: %q -> result = %v [%T], want %v [%T]", i+1, input.source, gotResult, gotResult, input.wantResult, input.wantResult)
			good = false
		}

		if gotErr != input.wantErr {
			if input.wantErr == nil || gotErr == nil || (gotErr.Error() != input.wantErr.Error()) {
				t.Errorf("%d: %q -> err = <%v>, want <%v>", i+1, input.source, gotErr, input.wantErr)
				good = false
			}
		}

		if good {
			succeeded++
		} else {
			failed++
		}
	}
	t.Log(fmt.Sprintf("test count: %d, succeeded count: %d, failed count: %d", len(inputs), succeeded, failed))
}
